@@ -4,6 +4,7 @@
 #include <type_traits>  //  std::enable_if, std::is_same, std::remove_const
 #include <vector>  //  std::vector
 #include <tuple>  //  std::tuple
+#include <utility>  //  std::move, std::forward
 #include <sstream>  //  std::stringstream
 
 #include "functional/cxx_universal.h"
@@ -14,9 +15,13 @@
 #include "constraints.h"
 #include "optional_container.h"
 #include "serializer_context.h"
+#include "serialize_result_type.h"
 #include "tags.h"
+#include "table_reference.h"
 #include "alias_traits.h"
 #include "expression.h"
+#include "column_pointer.h"
+#include "tags.h"
 #include "type_printer.h"
 #include "literal.h"
 
@@ -124,22 +129,22 @@ namespace sqlite_orm {
             using right_type = R;
             using result_type = Res;
 
-            left_type l;
-            right_type r;
+            left_type lhs;
+            right_type rhs;
 
             binary_condition() = default;
 
-            binary_condition(left_type l_, right_type r_) : l(std::move(l_)), r(std::move(r_)) {}
+            binary_condition(left_type l_, right_type r_) : lhs(std::move(l_)), rhs(std::move(r_)) {}
         };
 
         template<class T>
         SQLITE_ORM_INLINE_VAR constexpr bool is_binary_condition_v = is_base_of_template_v<T, binary_condition>;
 
         template<class T>
-        using is_binary_condition = polyfill::bool_constant<is_binary_condition_v<T>>;
+        struct is_binary_condition : polyfill::bool_constant<is_binary_condition_v<T>> {};
 
         struct and_condition_string {
-            operator std::string() const {
+            serialize_result_type serialize() const {
                 return "AND";
             }
         };
@@ -154,13 +159,8 @@ namespace sqlite_orm {
             using super::super;
         };
 
-        template<class L, class R>
-        and_condition_t<L, R> make_and_condition(L left, R right) {
-            return {std::move(left), std::move(right)};
-        }
-
         struct or_condition_string {
-            operator std::string() const {
+            serialize_result_type serialize() const {
                 return "OR";
             }
         };
@@ -175,13 +175,8 @@ namespace sqlite_orm {
             using super::super;
         };
 
-        template<class L, class R>
-        or_condition_t<L, R> make_or_condition(L left, R right) {
-            return {std::move(left), std::move(right)};
-        }
-
         struct is_equal_string {
-            operator std::string() const {
+            serialize_result_type serialize() const {
                 return "=";
             }
         };
@@ -219,8 +214,18 @@ namespace sqlite_orm {
             }
         };
 
+        template<class L, class R>
+        struct is_equal_with_table_t : negatable_t {
+            using left_type = L;
+            using right_type = R;
+
+            right_type rhs;
+
+            is_equal_with_table_t(right_type rhs) : rhs(std::move(rhs)) {}
+        };
+
         struct is_not_equal_string {
-            operator std::string() const {
+            serialize_result_type serialize() const {
                 return "!=";
             }
         };
@@ -248,7 +253,7 @@ namespace sqlite_orm {
         };
 
         struct greater_than_string {
-            operator std::string() const {
+            serialize_result_type serialize() const {
                 return ">";
             }
         };
@@ -276,7 +281,7 @@ namespace sqlite_orm {
         };
 
         struct greater_or_equal_string {
-            operator std::string() const {
+            serialize_result_type serialize() const {
                 return ">=";
             }
         };
@@ -303,8 +308,8 @@ namespace sqlite_orm {
             }
         };
 
-        struct lesser_than_string {
-            operator std::string() const {
+        struct less_than_string {
+            serialize_result_type serialize() const {
                 return "<";
             }
         };
@@ -313,10 +318,10 @@ namespace sqlite_orm {
          *  < operator object.
          */
         template<class L, class R>
-        struct lesser_than_t : binary_condition<L, R, lesser_than_string, bool>, negatable_t {
-            using self = lesser_than_t<L, R>;
+        struct less_than_t : binary_condition<L, R, less_than_string, bool>, negatable_t {
+            using self = less_than_t<L, R>;
 
-            using binary_condition<L, R, lesser_than_string, bool>::binary_condition;
+            using binary_condition<L, R, less_than_string, bool>::binary_condition;
 
             collate_t<self> collate_binary() const {
                 return {*this, collate_argument::binary};
@@ -331,8 +336,8 @@ namespace sqlite_orm {
             }
         };
 
-        struct lesser_or_equal_string {
-            operator std::string() const {
+        struct less_or_equal_string {
+            serialize_result_type serialize() const {
                 return "<=";
             }
         };
@@ -341,10 +346,10 @@ namespace sqlite_orm {
          *  <= operator object.
          */
         template<class L, class R>
-        struct lesser_or_equal_t : binary_condition<L, R, lesser_or_equal_string, bool>, negatable_t {
-            using self = lesser_or_equal_t<L, R>;
+        struct less_or_equal_t : binary_condition<L, R, less_or_equal_string, bool>, negatable_t {
+            using self = less_or_equal_t<L, R>;
 
-            using binary_condition<L, R, lesser_or_equal_string, bool>::binary_condition;
+            using binary_condition<L, R, less_or_equal_string, bool>::binary_condition;
 
             collate_t<self> collate_binary() const {
                 return {*this, collate_argument::binary};
@@ -559,12 +564,12 @@ namespace sqlite_orm {
 
         template<class T>
         SQLITE_ORM_INLINE_VAR constexpr bool is_order_by_v =
-            polyfill::disjunction_v<polyfill::is_specialization_of<T, order_by_t>,
-                                    polyfill::is_specialization_of<T, multi_order_by_t>,
-                                    polyfill::is_specialization_of<T, dynamic_order_by_t>>;
+            polyfill::disjunction<polyfill::is_specialization_of<T, order_by_t>,
+                                  polyfill::is_specialization_of<T, multi_order_by_t>,
+                                  polyfill::is_specialization_of<T, dynamic_order_by_t>>::value;
 
         template<class T>
-        using is_order_by = polyfill::bool_constant<is_order_by_v<T>>;
+        struct is_order_by : polyfill::bool_constant<is_order_by_v<T>> {};
 
         struct between_string {
             operator std::string() const {
@@ -626,7 +631,7 @@ namespace sqlite_orm {
         };
 
         template<class A, class T>
-        struct glob_t : condition_t, glob_string, internal::negatable_t {
+        struct glob_t : condition_t, glob_string, negatable_t {
             using self = glob_t<A, T>;
             using arg_t = A;
             using pattern_t = T;
@@ -826,185 +831,142 @@ namespace sqlite_orm {
         return {};
     }
 
-    template<class T, internal::satisfies<std::is_base_of, internal::negatable_t, T> = true>
-    internal::negated_condition_t<T> operator!(T arg) {
-        return {std::move(arg)};
+#ifdef SQLITE_ORM_WITH_CPP20_ALIASES
+    /**
+     *  Explicit FROM function. Usage:
+     *  `storage.select(&User::id, from<"a"_alias.for_<User>>());`
+     */
+    template<orm_refers_to_recordset auto... recordsets>
+    auto from() {
+        return from<internal::auto_decay_table_ref_t<recordsets>...>();
     }
+#endif
 
-    // Deliberately put operators for `expression_t` into the internal namespace
+    // Intentionally place operators for types classified as arithmetic or general operator arguments in the internal namespace
     // to facilitate ADL (Argument Dependent Lookup)
     namespace internal {
-        /**
-         *  Cute operators for columns
-         */
-        template<class T, class R>
-        lesser_than_t<T, R> operator<(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
+        template<
+            class T,
+            std::enable_if_t<polyfill::disjunction<std::is_base_of<negatable_t, T>, is_operator_argument<T>>::value,
+                             bool> = true>
+        negated_condition_t<T> operator!(T arg) {
+            return {std::move(arg)};
         }
 
-        template<class L, class T>
-        lesser_than_t<L, T> operator<(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<arithmetic_t, L>,
+                                                        std::is_base_of<arithmetic_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        less_than_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator<(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class T, class R>
-        lesser_or_equal_t<T, R> operator<=(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<arithmetic_t, L>,
+                                                        std::is_base_of<arithmetic_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        less_or_equal_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator<=(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class L, class T>
-        lesser_or_equal_t<L, T> operator<=(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<arithmetic_t, L>,
+                                                        std::is_base_of<arithmetic_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        greater_than_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator>(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class T, class R>
-        greater_than_t<T, R> operator>(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<arithmetic_t, L>,
+                                                        std::is_base_of<arithmetic_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        greater_or_equal_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator>=(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class L, class T>
-        greater_than_t<L, T> operator>(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<arithmetic_t, L>,
+                                                        std::is_base_of<arithmetic_t, R>,
+                                                        std::is_base_of<condition_t, L>,
+                                                        std::is_base_of<condition_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        is_equal_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator==(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class T, class R>
-        greater_or_equal_t<T, R> operator>=(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<arithmetic_t, L>,
+                                                        std::is_base_of<arithmetic_t, R>,
+                                                        std::is_base_of<condition_t, L>,
+                                                        std::is_base_of<condition_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        is_not_equal_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator!=(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class L, class T>
-        greater_or_equal_t<L, T> operator>=(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<polyfill::disjunction<std::is_base_of<condition_t, L>,
+                                                        std::is_base_of<condition_t, R>,
+                                                        is_operator_argument<L>,
+                                                        is_operator_argument<R>>::value,
+                                  bool> = true>
+        and_condition_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator&&(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class T, class R>
-        is_equal_t<T, R> operator==(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
+        template<class L,
+                 class R,
+                 std::enable_if_t<
+                     polyfill::disjunction<std::is_base_of<condition_t, L>, std::is_base_of<condition_t, R>>::value,
+                     bool> = true>
+        or_condition_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator||(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
 
-        template<class L, class T>
-        is_equal_t<L, T> operator==(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class T, class R>
-        is_not_equal_t<T, R> operator!=(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        is_not_equal_t<L, T> operator!=(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class T, class R>
-        conc_t<T, R> operator||(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        conc_t<L, T> operator||(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class L, class R>
-        conc_t<L, R> operator||(expression_t<L> l, expression_t<R> r) {
-            return {std::move(l.value), std::move(r.value)};
-        }
-
-        template<class R, class E, satisfies<std::is_base_of, conc_string, E> = true>
-        conc_t<E, R> operator||(E expr, R r) {
-            return {std::move(expr), std::move(r)};
-        }
-
-        template<class L, class E, satisfies<std::is_base_of, conc_string, E> = true>
-        conc_t<L, E> operator||(L l, E expr) {
-            return {std::move(l), std::move(expr)};
-        }
-
-        template<class T, class R>
-        add_t<T, R> operator+(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        add_t<L, T> operator+(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class L, class R>
-        add_t<L, R> operator+(expression_t<L> l, expression_t<R> r) {
-            return {std::move(l.value), std::move(r.value)};
-        }
-
-        template<class T, class R>
-        sub_t<T, R> operator-(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        sub_t<L, T> operator-(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class L, class R>
-        sub_t<L, R> operator-(expression_t<L> l, expression_t<R> r) {
-            return {std::move(l.value), std::move(r.value)};
-        }
-
-        template<class T, class R>
-        mul_t<T, R> operator*(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        mul_t<L, T> operator*(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class L, class R>
-        mul_t<L, R> operator*(expression_t<L> l, expression_t<R> r) {
-            return {std::move(l.value), std::move(r.value)};
-        }
-
-        template<class T, class R>
-        div_t<T, R> operator/(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        div_t<L, T> operator/(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class L, class R>
-        div_t<L, R> operator/(expression_t<L> l, expression_t<R> r) {
-            return {std::move(l.value), std::move(r.value)};
-        }
-
-        template<class T, class R>
-        mod_t<T, R> operator%(expression_t<T> expr, R r) {
-            return {std::move(expr.value), std::move(r)};
-        }
-
-        template<class L, class T>
-        mod_t<L, T> operator%(L l, expression_t<T> expr) {
-            return {std::move(l), std::move(expr.value)};
-        }
-
-        template<class L, class R>
-        mod_t<L, R> operator%(expression_t<L> l, expression_t<R> r) {
-            return {std::move(l.value), std::move(r.value)};
+        template<
+            class L,
+            class R,
+            std::enable_if_t<polyfill::conjunction<
+                                 polyfill::disjunction<std::is_base_of<conc_string, L>,
+                                                       std::is_base_of<conc_string, R>,
+                                                       is_operator_argument<L>,
+                                                       is_operator_argument<R>>,
+                                 // exclude conditions
+                                 polyfill::negation<polyfill::disjunction<std::is_base_of<condition_t, L>,
+                                                                          std::is_base_of<condition_t, R>>>>::value,
+                             bool> = true>
+        conc_t<unwrap_expression_t<L>, unwrap_expression_t<R>> operator||(L l, R r) {
+            return {get_from_expression(std::forward<L>(l)), get_from_expression(std::forward<R>(r))};
         }
     }
 
     template<class F, class O>
-    internal::using_t<O, F O::*> using_(F O::*p) {
-        return {p};
+    internal::using_t<O, F O::*> using_(F O::*field) {
+        return {field};
     }
     template<class T, class M>
-    internal::using_t<T, M> using_(internal::column_pointer<T, M> cp) {
-        return {std::move(cp)};
+    internal::using_t<T, M> using_(internal::column_pointer<T, M> field) {
+        return {std::move(field)};
     }
 
     template<class T>
@@ -1027,21 +989,49 @@ namespace sqlite_orm {
         return {std::move(o)};
     }
 
+#ifdef SQLITE_ORM_WITH_CPP20_ALIASES
+    template<orm_refers_to_recordset auto alias, class On>
+    auto left_join(On on) {
+        return left_join<internal::auto_decay_table_ref_t<alias>, On>(std::move(on));
+    }
+#endif
+
     template<class T, class O>
     internal::join_t<T, O> join(O o) {
         return {std::move(o)};
     }
 
+#ifdef SQLITE_ORM_WITH_CPP20_ALIASES
+    template<orm_refers_to_recordset auto alias, class On>
+    auto join(On on) {
+        return join<internal::auto_decay_table_ref_t<alias>, On>(std::move(on));
+    }
+#endif
+
     template<class T, class O>
     internal::left_outer_join_t<T, O> left_outer_join(O o) {
         return {std::move(o)};
     }
 
+#ifdef SQLITE_ORM_WITH_CPP20_ALIASES
+    template<orm_refers_to_recordset auto alias, class On>
+    auto left_outer_join(On on) {
+        return left_outer_join<internal::auto_decay_table_ref_t<alias>, On>(std::move(on));
+    }
+#endif
+
     template<class T, class O>
     internal::inner_join_t<T, O> inner_join(O o) {
         return {std::move(o)};
     }
 
+#ifdef SQLITE_ORM_WITH_CPP20_ALIASES
+    template<orm_refers_to_recordset auto alias, class On>
+    auto inner_join(On on) {
+        return inner_join<internal::auto_decay_table_ref_t<alias>, On>(std::move(on));
+    }
+#endif
+
     template<class T>
     internal::offset_t<T> offset(T off) {
         return {std::move(off)};
@@ -1062,36 +1052,18 @@ namespace sqlite_orm {
         return {std::move(lim), {std::move(offt.off)}};
     }
 
-    template<class L,
-             class R,
-             std::enable_if_t<polyfill::disjunction_v<std::is_base_of<internal::condition_t, L>,
-                                                      std::is_base_of<internal::condition_t, R>>,
-                              bool> = true>
-    auto operator&&(L l, R r) {
-        using internal::get_from_expression;
-        return internal::make_and_condition(std::move(get_from_expression(l)), std::move(get_from_expression(r)));
-    }
-
     template<class L, class R>
     auto and_(L l, R r) {
-        using internal::get_from_expression;
-        return internal::make_and_condition(std::move(get_from_expression(l)), std::move(get_from_expression(r)));
-    }
-
-    template<class L,
-             class R,
-             std::enable_if_t<polyfill::disjunction_v<std::is_base_of<internal::condition_t, L>,
-                                                      std::is_base_of<internal::condition_t, R>>,
-                              bool> = true>
-    auto operator||(L l, R r) {
-        using internal::get_from_expression;
-        return internal::make_or_condition(std::move(get_from_expression(l)), std::move(get_from_expression(r)));
+        using namespace ::sqlite_orm::internal;
+        return and_condition_t<unwrap_expression_t<L>, unwrap_expression_t<R>>{get_from_expression(std::forward<L>(l)),
+                                                                               get_from_expression(std::forward<R>(r))};
     }
 
     template<class L, class R>
     auto or_(L l, R r) {
-        using internal::get_from_expression;
-        return internal::make_or_condition(std::move(get_from_expression(l)), std::move(get_from_expression(r)));
+        using namespace ::sqlite_orm::internal;
+        return or_condition_t<unwrap_expression_t<L>, unwrap_expression_t<R>>{get_from_expression(std::forward<L>(l)),
+                                                                              get_from_expression(std::forward<R>(r))};
     }
 
     template<class T>
@@ -1144,6 +1116,11 @@ namespace sqlite_orm {
         return {std::move(l), std::move(r)};
     }
 
+    template<class L, class R>
+    internal::is_equal_with_table_t<L, R> is_equal(R rhs) {
+        return {std::move(rhs)};
+    }
+
     template<class L, class R>
     internal::is_not_equal_t<L, R> is_not_equal(L l, R r) {
         return {std::move(l), std::move(r)};
@@ -1175,22 +1152,40 @@ namespace sqlite_orm {
     }
 
     template<class L, class R>
-    internal::lesser_than_t<L, R> lesser_than(L l, R r) {
+    internal::less_than_t<L, R> less_than(L l, R r) {
+        return {std::move(l), std::move(r)};
+    }
+
+    /**
+     *  [Deprecation notice] This function is deprecated and will be removed in v1.10. Use the accurately named function `less_than(...)` instead.
+     */
+    template<class L, class R>
+    [[deprecated("Use the accurately named function `less_than(...)` instead")]] internal::less_than_t<L, R>
+    lesser_than(L l, R r) {
         return {std::move(l), std::move(r)};
     }
 
     template<class L, class R>
-    internal::lesser_than_t<L, R> lt(L l, R r) {
+    internal::less_than_t<L, R> lt(L l, R r) {
         return {std::move(l), std::move(r)};
     }
 
     template<class L, class R>
-    internal::lesser_or_equal_t<L, R> lesser_or_equal(L l, R r) {
+    internal::less_or_equal_t<L, R> less_or_equal(L l, R r) {
+        return {std::move(l), std::move(r)};
+    }
+
+    /**
+     *  [Deprecation notice] This function is deprecated and will be removed in v1.10. Use the accurately named function `less_or_equal(...)` instead.
+     */
+    template<class L, class R>
+    [[deprecated("Use the accurately named function `less_or_equal(...)` instead")]] internal::less_or_equal_t<L, R>
+    lesser_or_equal(L l, R r) {
         return {std::move(l), std::move(r)};
     }
 
     template<class L, class R>
-    internal::lesser_or_equal_t<L, R> le(L l, R r) {
+    internal::less_or_equal_t<L, R> le(L l, R r) {
         return {std::move(l), std::move(r)};
     }
 
@@ -1222,8 +1217,8 @@ namespace sqlite_orm {
      * Example: storage.get_all<Singer>(multi_order_by(order_by(&Singer::name).asc(), order_by(&Singer::gender).desc())
      */
     template<class... Args>
-    internal::multi_order_by_t<Args...> multi_order_by(Args&&... args) {
-        return {std::make_tuple(std::forward<Args>(args)...)};
+    internal::multi_order_by_t<Args...> multi_order_by(Args... args) {
+        return {{std::forward<Args>(args)...}};
     }
 
     /**
