@@ -1305,6 +1305,22 @@ int main(int, char**) {
                  << endl;
         }
     }
+    {
+        //  SELECT employee_id, first_name, last_name, salary, (SELECT AVG(salary) FROM employees), salary > (SELECT AVG(salary) FROM employees)
+        //  FROM "employees";
+        auto rows = storage.select(columns(&Employee::id,
+                                           &Employee::firstName,
+                                           &Employee::lastName,
+                                           &Employee::salary,
+                                           select(avg(&Employee::salary)),
+                                           greater_than(&Employee::salary, select(avg(&Employee::salary)))));
+        cout << "employee_id  first_name  last_name   salary      avg_salary  salary>avg" << endl;
+        cout << "-----------  ----------  ----------  ----------  ----------  ----------" << endl;
+        for(auto& row: rows) {
+            cout << std::get<0>(row) << '\t' << std::get<1>(row) << '\t' << std::get<2>(row) << '\t' << std::get<3>(row)
+                 << '\t' << std::get<4>(row) << '\t' << std::get<5>(row) << endl;
+        }
+    }
     {
         //  SELECT first_name, last_name, department_id
         //  FROM employees
@@ -1346,6 +1362,21 @@ int main(int, char**) {
         //  WHERE salary >(SELECT AVG(salary)
         //      FROM employees
         //      WHERE department_id = e.department_id);
+#ifdef SQLITE_ORM_WITH_CPP20_ALIASES
+        constexpr orm_table_alias auto e = "e"_alias.for_<Employee>();
+        auto rows = storage.select(
+            columns(e->*&Employee::lastName, e->*&Employee::salary, e->*&Employee::departmentId),
+            from<e>(),
+            where(greater_than(e->*&Employee::salary,
+                               select(avg(&Employee::salary),
+                                      from<Employee>(),
+                                      where(is_equal(&Employee::departmentId, e->*&Employee::departmentId))))));
+        cout << "last_name   salary      department_id" << endl;
+        cout << "----------  ----------  -------------" << endl;
+        for(auto& row: rows) {
+            cout << std::get<0>(row) << '\t' << std::get<1>(row) << '\t' << std::get<2>(row) << endl;
+        }
+#else
         using als = alias_e<Employee>;
         auto rows = storage.select(
             columns(alias_column<als>(&Employee::lastName),
@@ -1362,6 +1393,7 @@ int main(int, char**) {
         for(auto& row: rows) {
             cout << std::get<0>(row) << '\t' << std::get<1>(row) << '\t' << std::get<2>(row) << endl;
         }
+#endif
     }
     {
         //  SELECT first_name, last_name, employee_id, job_id
@@ -1372,10 +1404,10 @@ int main(int, char**) {
         auto rows =
             storage.select(columns(&Employee::firstName, &Employee::lastName, &Employee::id, &Employee::jobId),
                            from<Employee>(),
-                           where(lesser_or_equal(1,
-                                                 select(count<JobHistory>(),
-                                                        from<JobHistory>(),
-                                                        where(is_equal(&Employee::id, &JobHistory::employeeId))))));
+                           where(less_or_equal(1,
+                                               select(count<JobHistory>(),
+                                                      from<JobHistory>(),
+                                                      where(is_equal(&Employee::id, &JobHistory::employeeId))))));
         cout << "first_name  last_name   employee_id  job_id" << endl;
         cout << "----------  ----------  -----------  ----------" << endl;
         for(auto& row: rows) {
