@@ -1,38 +1,34 @@
 #pragma once
-
 #include "../../dev/functional/start_macros.h"
+// pull in the SQLite3 configuration early, such that version and feature macros are globally available in sqlite_orm
+#include "../../dev/functional/sqlite3_config.h"
 // though each header is required to include everything it needs
 // we include the configuration and all underlying c++ core features in order to make it universally available
 #include "../../dev/functional/config.h"
 #include "../../dev/type_traits.h"
-#include "../../dev/error_code.h"
-#include "../../dev/type_printer.h"
 #include "../../dev/collate_argument.h"
 #include "../../dev/constraints.h"
 #include "../../dev/type_is_nullable.h"
 #include "../../dev/operators.h"
-#include "../../dev/column.h"
-#include "../../dev/field_printer.h"
+#include "../../dev/schema/column.h"
 #include "../../dev/conditions.h"
 #include "../../dev/alias.h"
 #include "../../dev/core_functions.h"
 #include "../../dev/select_constraints.h"
 #include "../../dev/table_info.h"
-#include "../../dev/triggers.h"
-#include "../../dev/statement_finalizer.h"
-#include "../../dev/arithmetic_tag.h"
-#include "../../dev/pointer_value.h"
+#include "../../dev/schema/triggers.h"
 #include "../../dev/statement_binder.h"
 #include "../../dev/row_extractor.h"
 #include "../../dev/sync_schema_result.h"
-#include "../../dev/index.h"
+#include "../../dev/schema/index.h"
 #include "../../dev/rowid.h"
-#include "../../dev/table.h"
+#include "../../dev/schema/table.h"
 #include "../../dev/storage_impl.h"
 #include "../../dev/default_value_extractor.h"
 #include "../../dev/storage.h"
 #include "../../dev/get_prepared_statement.h"
 #include "../../dev/carray.h"
-#include "../../dev/dbstat.h"
+#include "../../dev/sqlite_schema_table.h"
+#include "../../dev/eponymous_vtabs/dbstat.h"
 #include "../../dev/interface_definitions.h"
 #include "../../dev/functional/finish_macros.h"
