@@ -19,4 +19,11 @@ TEST_CASE("alias extractor") {
         REQUIRE(alias_extractor<alias_a<User>>::extract() == "a");
         REQUIRE(alias_extractor<alias_a<User>>::as_alias() == "a");
     }
+#if(SQLITE_VERSION_NUMBER >= 3008003) && defined(SQLITE_ORM_WITH_CTE)
+    SECTION("cte moniker") {
+        using cte_1 = decltype(1_ctealias);
+        REQUIRE(alias_extractor<cte_1>::extract() == "1");
+        REQUIRE(alias_extractor<cte_1>::as_alias() == "");
+    }
+#endif
 }
